@@ -486,8 +486,12 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
     def get_unretraced(self):
         '''Return an ID set of all crashes which have not been retraced yet and
         which happened on the current host architecture.'''
-        bugs = self.lp_distro.searchTasks(tags=self.arch_tag)
-        return id_set(bugs)
+        try:
+            bugs = self.lp_distro.searchTasks(tags=self.arch_tag)
+            return id_set(bugs)
+        except Exception, e:
+            apport.error('connecting to Launchpad failed: %s', str(e))
+            sys.exit(99) # transient error
 
     def get_dup_unchecked(self):
         '''Return an ID set of all crashes which have not been checked for
@@ -497,8 +501,12 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
         Python, since they do not need to be retraced. It should not return
         bugs that are covered by get_unretraced().'''
         
-        bugs = self.lp_distro.searchTasks(tags='need-duplicate-check')
-        return id_set(bugs)
+        try:
+            bugs = self.lp_distro.searchTasks(tags='need-duplicate-check')
+            return id_set(bugs)
+        except Exception, e:
+            apport.error('connecting to Launchpad failed: %s', str(e))
+            sys.exit(99) # transient error
 
     def get_unfixed(self):
         '''Return an ID set of all crashes which are not yet fixed.
@@ -621,6 +629,11 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
             master = self.launchpad.bugs[master_id]
             if master.duplicate_of:
                 master = master.duplicate_of
+                if master.id == id:
+                    # this happens if the bug was manually duped to a newer one
+                    apport.warning('Bug %i was manually marked as a dupe of newer bug %i, not closing as duplicate', 
+                            id, master_id)
+                    return
             
             for a in bug.attachments:
                 if a.title in ('CoreDump.gz', 'Stacktrace.txt',
@@ -748,7 +761,7 @@ in a dependent package.' % master,
         #FIXME: this entire function is an ugly Ubuntu specific hack until LP
         #gets a real crash db; see https://wiki.ubuntu.com/CrashReporting
 
-        if report.get('DistroRelease', '').split()[0] != 'Ubuntu':
+        if 'DistroRelease' in report and report['DistroRelease'].split()[0] != 'Ubuntu':
             return # only Ubuntu bugs are filed private
         
         #use a url hack here, it is faster
@@ -1563,11 +1576,9 @@ NameError: global name 'weird' is not defined'''
             # package name
             b = self.crashdb.launchpad.bugs[python_report]
             t = b.bug_tasks[0]
-            t.target = self.crashdb.launchpad.distributions['ubuntu'].getSourcePackage(name='bash')
-            t.status = 'Invalid'
+            t.target = self.crashdb.launchpad.distributions['ubuntu']
             t.lp_save()
             b.addTask(target=self.crashdb.launchpad.projects['coreutils'])
-            b.addTask(target=self.crashdb.launchpad.distributions['ubuntu'])
 
             self.crashdb._mark_dup_checked(python_report, self.ref_report)
 
@@ -1582,14 +1593,10 @@ NameError: global name 'weird' is not defined'''
             self.assertEqual(b.bug_tasks[0].status, 'New')
 
             # package-less distro task should have package name fixed
-            self.assertEqual(b.bug_tasks[2].bug_target_name, 'coreutils (Ubuntu)')
-            self.assertEqual(b.bug_tasks[2].status, 'New')
+            self.assertEqual(b.bug_tasks[1].bug_target_name, 'coreutils (Ubuntu)')
+            self.assertEqual(b.bug_tasks[1].status, 'New')
 
-            # invalid bash task should be unmodified
-            self.assertEqual(b.bug_tasks[1].bug_target_name, 'bash (Ubuntu)')
-            self.assertEqual(b.bug_tasks[1].status, 'Invalid')
-
-            # the invalid task should not confuse get_fixed_version()
+            # should not confuse get_fixed_version()
             self.assertEqual(self.crashdb.get_fixed_version(python_report),
                     None)
 
