@@ -32,23 +32,29 @@ _path_key_trans = string.maketrans('#/-_+ ','....._')
 def path_to_key(path):
     '''Generate a valid report key name from a file path.
         
-    This will meet apport's restrictions on the characters used in keys.
+    This will replace invalid punctuation symbols with valid ones.
     '''
     return path.translate(_path_key_trans)
 
-def attach_file_if_exists(report, path, key=None):
-    '''Attach file contents if file exists.'''
+def attach_file_if_exists(report, path, key=None, overwrite=True):
+    '''Attach file contents if file exists.
 
+    If key is not specified, the key name will be derived from the file
+    name with path_to_key().
+
+    If overwrite is True, an existing key will be updated. If it is False, a
+    new key with '_' appended will be added instead.
+    '''
     if not key:
         key = path_to_key(path)
 
     if os.path.exists(path):
-        attach_file(report, path, key)
+        attach_file(report, path, key, overwrite)
 
 def read_file(path):
     '''Return the contents of the specified path. 
         
-    Upon error, this will deliver a a text representation of the error,
+    Upon error, this will deliver a text representation of the error,
     instead of failing.
     '''
     try:
@@ -56,24 +62,28 @@ def read_file(path):
     except Exception as e:
         return 'Error: ' + str(e)
 
-def attach_file(report, path, key=None):
+def attach_file(report, path, key=None, overwrite=True):
     '''Attach a file to the report.
 
     If key is not specified, the key name will be derived from the file
     name with path_to_key().
+
+    If overwrite is True, an existing key will be updated. If it is False, a
+    new key with '_' appended will be added instead.
     '''
     if not key:
         key = path_to_key(path)
 
     # Do not clobber existing keys
-    while report.has_key(key):
-        key += "_"
+    if not overwrite:
+        while report.has_key(key):
+            key += '_'
     report[key] = read_file(path)
 
 def attach_dmesg(report):
     '''Attach information from the kernel ring buffer (dmesg).
 
-    This won't overwite already existing information.
+    This will not overwrite already existing information.
     '''
     try:
         if not report.get('BootDmesg', '').strip():
@@ -210,6 +220,10 @@ def attach_alsa(report):
             + glob.glob('/dev/snd/*')
             + glob.glob('/dev/seq*') )
 
+    if os.path.exists('/usr/bin/pacmd'):
+        report['PulseSinks'] = command_output(['pacmd', 'list-sinks'])
+        report['PulseSources'] = command_output(['pacmd', 'list-sources'])
+
     attach_dmi(report)
     attach_dmesg(report)
 
@@ -258,7 +272,7 @@ def _root_command_prefix():
                     stderr=subprocess.PIPE) == 0 and \
             subprocess.call(['pgrep', '-x', '-u', str(os.getuid()), 'ksmserver'],
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-        prefix = ['kdesudo', '--']
+        prefix = ['kdesudo', '--desktop', '/usr/share/applications/apport-kde-mime.desktop', '--']
     elif os.getenv('DISPLAY') and \
             subprocess.call(['which', 'gksu'], stdout=subprocess.PIPE,
                     stderr=subprocess.PIPE) == 0 and \
@@ -318,19 +332,28 @@ def attach_root_command_outputs(report, command_map):
         # now read back the individual outputs
         for keyname in command_map:
             f = open(os.path.join(workdir, keyname))
-            report[keyname] = f.read()
+            buf = f.read().strip()
+            if buf:
+                report[keyname] = buf
             f.close()
     finally:
         shutil.rmtree(workdir)
 
 def recent_syslog(pattern):
     '''Extract recent messages from syslog which match a regex.
-        
+
+    pattern should be a "re" object.
+    '''
+    return recent_logfile('/var/log/syslog', pattern)
+
+def recent_logfile(logfile, pattern):
+    '''Extract recent messages from a logfile which match a regex.
+
     pattern should be a "re" object.
     '''
     lines = ''
     try:
-        for line in open('/var/log/syslog'):
+        for line in open(logfile):
             if pattern.search(line):
                 lines += line
     except IOError:
@@ -469,7 +492,7 @@ def attach_network(report):
 def attach_wifi(report):
     '''Attach wireless (WiFi) network information to report.'''
 
-    report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel)(\[\d+\])?:'))
+    report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel|wpa_supplicant)(\[\d+\])?:'))
     report['IwConfig'] = re.sub('Encryption key:(.*)', 'Encryption key: <hidden>', command_output(['iwconfig']))
     report['RfKill'] = command_output(['rfkill', 'list'])
     report['CRDA'] = command_output(['iw', 'reg', 'get'])
@@ -502,6 +525,31 @@ def attach_printing(report):
         'cupsys-driver-gutenprint', 'gimp-gutenprint', 'gutenprint-doc',
         'gutenprint-locales', 'system-config-printer-common', 'kdeprint')
 
+def attach_mac_events(report):
+    '''Attach MAC information and events to the report.'''
+
+    mac_regex = 'audit\(|apparmor|selinux|security'
+    mac_re = re.compile(mac_regex, re.IGNORECASE)
+    aa_denied_regex = 'apparmor="DENIED"'
+    aa_denied_re = re.compile(aa_denied_regex, re.IGNORECASE)
+
+    if os.path.exists('/var/log/kern.log'):
+        report['KernLog'] = recent_logfile('/var/log/kern.log', mac_re)
+    elif os.path.exists('/var/log/messages'):
+        report['KernLog'] = recent_logfile('/var/log/messages', mac_re)
+
+    if os.path.exists('/var/run/auditd.pid'):
+        attach_root_command_outputs(report, {'AuditLog': 'egrep "' + mac_regex + '" /var/log/audit/audit.log'})
+
+    attach_file(report, '/proc/version_signature', 'ProcVersionSignature')
+    attach_file(report, '/proc/cmdline', 'ProcCmdline')
+
+    if re.search(aa_denied_re, report.get('KernLog', '')) or re.search(aa_denied_re, report.get('AuditLog', '')):
+        tags = report.get('Tags', '')
+        if tags:
+            tags += ' '
+        report['Tags'] = tags + 'apparmor'
+
 def attach_related_packages(report, packages):
     '''Attach version information for related packages
 
@@ -741,8 +789,69 @@ if __name__ == '__main__':
             self.assertEqual(report['BootDmesg'], 'existingboot')
             self.assertEqual(report['CurrentDmesg'], 'existingcurrent')
 
+        def test_attach_file(self):
+            '''attach_file()'''
+
+            with open('/etc/motd') as f:
+                motd_contents = f.read().strip()
+            with open('/etc/issue') as f:
+                issue_contents = f.read().strip()
+
+            # default key name
+            report = {}
+            attach_file(report, '/etc/motd')
+            self.assertEqual(report.keys(), ['.etc.motd'])
+            self.assertEqual(report['.etc.motd'], motd_contents)
+
+            # custom key name
+            report = {}
+            attach_file(report, '/etc/motd', 'Motd')
+            self.assertEqual(report.keys(), ['Motd'])
+            self.assertEqual(report['Motd'], motd_contents)
+
+            # nonexisting file
+            report = {}
+            attach_file(report, '/nonexisting')
+            self.assertEqual(report.keys(), ['.nonexisting'])
+            self.assertTrue(report['.nonexisting'].startswith('Error: '))
+
+            # existing key
+            report = {}
+            attach_file(report, '/etc/motd')
+            attach_file(report, '/etc/motd')
+            self.assertEqual(report.keys(), ['.etc.motd'])
+            self.assertEqual(report['.etc.motd'], motd_contents)
+
+            attach_file(report, '/etc/issue', '.etc.motd', overwrite=False)
+            self.assertEqual(sorted(report.keys()), ['.etc.motd', '.etc.motd_'])
+            self.assertEqual(report['.etc.motd'], motd_contents)
+            self.assertEqual(report['.etc.motd_'], issue_contents)
+
+        def test_attach_file_if_exists(self):
+            '''attach_file_if_exists()'''
+
+            with open('/etc/motd') as f:
+                motd_contents = f.read().strip()
+
+            # default key name
+            report = {}
+            attach_file_if_exists(report, '/etc/motd')
+            self.assertEqual(report.keys(), ['.etc.motd'])
+            self.assertEqual(report['.etc.motd'], motd_contents)
+
+            # custom key name
+            report = {}
+            attach_file_if_exists(report, '/etc/motd', 'Motd')
+            self.assertEqual(report.keys(), ['Motd'])
+            self.assertEqual(report['Motd'], motd_contents)
+
+            # nonexisting file
+            report = {}
+            attach_file_if_exists(report, '/nonexisting')
+            self.assertEqual(report.keys(), [])
+
         def test_no_crashes(self):
-            '''functions do not crash (very shallow'''
+            '''functions do not crash (very shallow)'''
 
             report = {}
             attach_hardware(report)
