@@ -15,7 +15,7 @@ import fnmatch, glob, atexit, traceback, errno
 import xml.dom, xml.dom.minidom
 from xml.parsers.expat import ExpatError
 
-from problem_report import ProblemReport
+import problem_report
 import apport
 import apport.fileutils
 from apport.packaging_impl import impl as packaging
@@ -108,7 +108,10 @@ def _check_bug_pattern(report, pattern):
                 c.childNodes[0].nodeType == xml.dom.Node.TEXT_NODE:
                 regexp = c.childNodes[0].nodeValue.encode('UTF-8')
                 try:
-                    if not re.search(regexp, report[key]):
+                    v = report[key]
+                    if isinstance(v, problem_report.CompressedValue):
+                        v = v.get_value()
+                    if not re.search(regexp, v):
                         return None
                 except:
                     return None
@@ -142,7 +145,7 @@ def _dom_remove_space(node):
 # Report class
 #
 
-class Report(ProblemReport):
+class Report(problem_report.ProblemReport):
     '''A problem report specific to apport (crash or bug).
 
     This class wraps a standard ProblemReport and adds methods for collecting
@@ -157,7 +160,7 @@ class Report(ProblemReport):
         If the report is attached to a process ID, this should be set in
         self.pid, so that e. g. hooks can use it to collect additional data.
         '''
-        ProblemReport.__init__(self, type, date)
+        problem_report.ProblemReport.__init__(self, type, date)
         self.pid = None
 
     def _pkg_modified_suffix(self, package):
@@ -457,7 +460,7 @@ class Report(ProblemReport):
                 os.unlink(core)
         return ret
 
-    def add_gdb_info(self, debugdir=None):
+    def add_gdb_info(self, rootdir=None):
         '''Add information from gdb.
 
         This requires that the report has a CoreDump and an
@@ -470,8 +473,10 @@ class Report(ProblemReport):
           inclusion into bug reports and easier processing
         - AssertionMessage: Value of __abort_msg, if present
 
-        The optional debugdir can specify an alternative debug symbol root
-        directory.
+        The optional rootdir can specify a root directory which has the
+        executable, libraries, and debug symbols. This does not require
+        chroot() or root privileges, it just instructs gdb to search for the
+        files there.
         '''
         if not self.has_key('CoreDump') or not self.has_key('ExecutablePath'):
             return
@@ -500,10 +505,12 @@ class Report(ProblemReport):
                           }
 
             command = ['gdb', '--batch']
-            if debugdir:
-                command += ['--ex', 'set debug-file-directory ' + debugdir]
-            command += ['--ex', 'file ' + self.get('InterpreterPath',
-                self['ExecutablePath']), '--ex', 'core-file ' + core]
+            executable = self.get('InterpreterPath', self['ExecutablePath'])
+            if rootdir:
+                command += ['--ex', 'set debug-file-directory %s/usr/lib/debug' % rootdir,
+                            '--ex', 'set solib-absolute-prefix ' + rootdir]
+                executable = rootdir + '/' + executable
+            command += ['--ex', 'file ' + executable, '--ex', 'core-file ' + core]
             # limit maximum backtrace depth (to avoid looped stacks)
             command += ['--ex', 'set backtrace limit 2000']
             value_keys = []
@@ -512,7 +519,7 @@ class Report(ProblemReport):
                 value_keys.append(name)
                 command += ['--ex', 'p -99', '--ex', cmd]
 
-            assert os.path.exists(self.get('InterpreterPath', self['ExecutablePath']))
+            assert os.path.exists(executable)
 
             # call gdb
             try:
@@ -905,7 +912,7 @@ class Report(ProblemReport):
             path = os.path.basename(self['ExecutablePath'])
             last_line = trace[-1]
             exception = last_line.split(':')[0]
-            m = re.match('^%s: (.+)$' % exception, last_line)
+            m = re.match('^%s: (.+)$' % re.escape(exception), last_line)
             if m:
                 message = m.group(1)
             else:
@@ -1429,6 +1436,15 @@ sys.stdin.readline()
         self.assertEqual(pr['InterpreterPath'], '/usr/bin/python')
         self.assertEqual(pr['ExecutablePath'], '/bin/bash')
 
+        # python script with a versioned interpreter
+        pr = Report()
+        pr['ExecutablePath'] = '/usr/bin/python2.7'
+        pr['ProcStatus'] = 'Name:\tbash'
+        pr['ProcCmdline'] = '/usr/bin/python\0/bin/bash'
+        pr._check_interpreted()
+        self.assertEqual(pr['InterpreterPath'], '/usr/bin/python2.7')
+        self.assertEqual(pr['ExecutablePath'], '/bin/bash')
+
     @classmethod
     def _generate_sigsegv_report(klass, file=None, signal='11', code='''
 int f(x) {
@@ -1773,6 +1789,11 @@ $0.bin 2>/dev/null
         self.assertEqual(r_coreutils.search_bug_patterns(patterns.name), 'http://bugtracker.net/bugs/3')
         self.assertEqual(r_bazaar.search_bug_patterns(patterns.name), 'http://bugtracker.net/bugs/5')
 
+        # also works for CompressedValues
+        r_bash_compressed = r_bash.copy()
+        r_bash_compressed['Foo'] = problem_report.CompressedValue('bazaar')
+        self.assertEqual(r_bash_compressed.search_bug_patterns(patterns.name), 'http://bugtracker.net/bugs/1')
+
         # negative match cases
         r_bash['Package'] = 'bash-static 1-2'
         self.assertEqual(r_bash.search_bug_patterns(patterns.name), None)
@@ -2130,9 +2151,9 @@ order (MRO) for bases GObject, CanvasGroupableIface, CanvasGroupable'''
         report['Traceback'] = '''Traceback (most recent call last):
   File "/x/foo.py", line 242, in setup_chooser
     raise "Moo"
-Moo'''
+Mo?o[a-1]'''
 
-        self.assertEqual(report.standard_title(), 'apport-gtk crashed with Moo in setup_chooser()')
+        self.assertEqual(report.standard_title(), 'apport-gtk crashed with Mo?o[a-1] in setup_chooser()')
 
         # Python crash with custom message with newlines (LP #190947)
         report = Report()
